# -*- coding: utf-8 -*-

#  Developed by CQ Inversiones SAS. Copyright ©. 2019 - 2023. All rights reserved.
#  Desarrollado por CQ Inversiones SAS. Copyright ©. 2019 - 2023. Todos los derechos reservado

# ****************************************************************
# IDE:          PyCharm
# Developed by: macercha
# Date:         17/01/23 10:39 PM
# Project:      CFHL Transactional Backend
# Module Name:  send_code
# Description:
# ****************************************************************
import string
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.utils.translation import gettext_lazy as _
from django.utils.translation import gettext as __
from zibanu.django.utils import Email


class SendCode:
    def __init__(self, to: str, action: str, context: dict = None):
        self.__action = action
        self.__to = to
        self.__context = context
        self.__subject_prefix = _(string.capwords(self.__action.replace("_", " ")))
        self.__email_subject = self.__subject_prefix + " / " + __("Authorization Code")
        self.__email = Email(subject=self.__email_subject, from_email=settings.ZB_MAIL_DEFAULT_FROM, to=[to])

    def load_templates(self, context: dict = None) -> bool:
        b_return = False
        # Override context private property
        if context is not None:
            self.__context = context
        if self.__context is not None:
            template_text = self.__action.lower() + ".txt"
            template_html = self.__action.lower() + ".html"
            try:
                self.__email.set_html_template(template_html, context=self.__context)
                self.__email.set_text_template(template_text, context=self.__context)
            except TemplateDoesNotExist:
                self.__email.set_html_template("default.html", context=self.__context)
                self.__email.set_text_template("default.txt", context=self.__context)
            b_return = True
        return b_return

    def send(self):
        self.__email.send()





# -*- coding: utf-8 -*-

#  Developed by CQ Inversiones SAS. Copyright ©. 2019 - 2022. All rights reserved.
#  Desarrollado por CQ Inversiones SAS. Copyright ©. 2019 - 2022. Todos los derechos reservado

# ****************************************************************
# IDE:          PyCharm
# Developed by: macercha
# Date:         19/12/22 9:19 PM
# Project:      CFHL Transactional Backend
# Module Name:  apps
# Description:
# ****************************************************************
from django.conf import settings
from django.apps import AppConfig


class CFHLAuth(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'oasis_auth'

    def ready(self):
        settings.OASIS_AUTH_CODE_TIMEOUT = getattr(settings, "OASIS_AUTH_CODE_TIMEOUT", 2)



# -*- coding: utf-8 -*-

#  Developed by CQ Inversiones SAS. Copyright ©. 2019 - 2023. All rights reserved.
#  Desarrollado por CQ Inversiones SAS. Copyright ©. 2019 - 2023. Todos los derechos reservado

# ****************************************************************
# IDE:          PyCharm
# Developed by: macercha
# Date:         7/01/23 4:03 PM
# Project:      CFHL Transactional Backend
# Module Name:  models
# Description:
# ****************************************************************
from django.contrib.auth.models import User
from django.core.validators import MinValueValidator
from django.utils.translation import gettext_lazy as _
from oasis.lib.choices import CustomerType
from oasis.models import DocumentType
from oasis_auth.lib import managers
from zibanu.django.db import models


class UserProfile(models.Model):
    """
    Entity class for extend user profile definition combined with oasis module
    """
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name="profile", related_query_name="user")
    document_type = models.ForeignKey(DocumentType, on_delete=models.PROTECT, blank=False, null=False,
                                      related_name="profile", related_query_name="document_types")
    document_id = models.BigIntegerField(blank=False, null=False, validators=[
        MinValueValidator(limit_value=1, message=_("The document id is invalid."))])
    location = models.CharField(max_length=150, blank=True, null=False)
    address = models.CharField(max_length=100, blank=True, null=True)
    phone = models.CharField(max_length=30, blank=True, null=True)
    mobile = models.CharField(max_length=30, blank=True, null=True)
    type = models.IntegerField(choices=CustomerType.choices, default=CustomerType.ANYTHING, blank=False, null=False)
    enabled = models.BooleanField(default=True, blank=False, null=False)
    # Default Manager
    objects = managers.UserProfile()

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=("document_type", "document_id"), name="UNQ_UserProfile_Document")
        ]



# -*- coding: utf-8 -*-

#  Developed by CQ Inversiones SAS. Copyright ©. 2019 - 2023. All rights reserved.
#  Desarrollado por CQ Inversiones SAS. Copyright ©. 2019 - 2023. Todos los derechos reservado

# ****************************************************************
# IDE:          PyCharm
# Developed by: macercha
# Date:         18/01/23 10:12 AM
# Project:      CFHL Transactional Backend
# Module Name:  auth_actions
# Description:
# ****************************************************************
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from oasis_auth.models import UserProfile
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework_simplejwt.settings import api_settings
from rest_framework_simplejwt.serializers import TokenObtainSerializer
from typing import Any


class AuthActions:
    """
    Class to encapsulate the different actions to be executed for auth package
    depending on action required.
    """
    def __init__(self, data: dict):
        """
        Initialization method
        :param data: data dictionary with at least "data" and "action" keys.
        """
        if data is not None and {"data", "action"} <= data.keys():
            self.__data = data.get("data")
            self.__action = data.get("action")
            self.__password = data.get("password")
        else:
            raise ValidationError(_("Invalid initialization data class."))

    def __get_method(self):
        return "_" + self.__action

    def do_action(self) -> Any:
        if hasattr(self, self.__get_method()):
            # Get the method of class to invoke
            method = getattr(self, self.__get_method())
            # Invoke method
            return method()
        else:
            raise ValidationError(_("Action does not have an associated method."))

    def _register(self) -> Any:
        UserProfile.objects.create_from_customer_data(self.__data, self.__password)
        return None

    def _login(self) -> Any:
        token_data = RefreshToken.for_user(self.__data)
        data_return = {
            "token": str(token_data.access_token),
            "payload": token_data.payload,
            "user": {
                "full_name": self.__data.get_full_name(),
                "document_id": self.__data.profile.document_id,
                "type": self.__data.profile.get_type_display()
            }
        }
        return data_return






